import json
import time

import pandas as pd
from notecoin.base.tables.strategy import StrategyTable
from notecoin.task import AccountTask, BaseTask, MarketTask, Ticker24HTask
from notecoin.utils import logger


class StrategyTask(BaseTask):

    def __init__(self, *args, **kwargs):
        super(StrategyTask, self).__init__(*args, **kwargs)
        self.table = StrategyTable(db_suffix=self.exchange.name)
        self.table.create()

    def buy_auto(self):
        account = pd.read_sql(sql=f"select * from {AccountTask.table_name}", con=self.engine.connect())
        curr = 0
        for symbol in json.loads(account.to_json(orient='records')):
            if symbol['symbol'] == 'BUSD':
                curr = symbol['free']
        if curr < 12:
            logger.info("account<12")
            return
        market = pd.read_sql(sql=f"select * from {MarketTask.table_name}", con=self.engine.connect())
        ticker24h = pd.read_sql(sql=f"select * from {Ticker24HTask.table_name}", con=self.engine.connect())
        ticker24h = ticker24h.sort_values(['quoteVolume'], ascending=False).reset_index(drop=True)
        for symbol_info in json.loads(ticker24h.to_json(orient='records')):
            symbol = symbol_info['symbol']
            if not symbol.endswith('BUSD'):
                continue

            tmp = market[market['id'] == symbol]
            print(f'{symbol},{len(tmp)}')
            if len(tmp) == 1:
                self.buy_market(tmp['symbol'].values[0])

        logger.info("done")

    def sell_auto(self):
        data = pd.read_sql(f"select * from {self.table.table_name}", con=self.engine.connect())
        for row in json.loads(data.to_json(orient='records')):
            symbol = row['symbol']
            if symbol is None:
                continue
            price = self.current_price(symbol)

            buy_info = json.loads(row['buy_json'])
            buy_price = buy_info['price']
            amount = buy_info['amount']
            timestamp = buy_info['timestamp']
            if time.time() * 1000 - timestamp > 30 * 60 * 1000:
                logger.info(f"out of time,sell {buy_price} vs {price}")
                self.sell_market(row['id'], symbol, amount)
            elif abs((buy_price - price) / price) > 0.005:
                logger.info(f"buy price {buy_price} vs {price}")
                self.sell_market(row['id'], symbol, amount)

    def buy_market(self, symbol, total=12):
        price = self.current_price(symbol)
        if price == 0:
            return
        logger.info(f"buy {symbol}")

        amount = total / price
        buy_json = self.exchange.create_order(symbol, 'market', 'buy', amount)
        value = {
            "status": 2,
            "ext_json": {},
            "symbol": symbol,
            "amount": amount,
            "buy_json": buy_json,
        }

        self.table.upsert(values=[value])

    def sell_market(self, id, symbol, amount):
        logger.info(f"sell {symbol}")

        sell_json = self.exchange.create_order(symbol, 'market', 'sell', amount)
        value = {
            "id": id,
            "sell_json": sell_json,
        }

        self.table.upsert(values=[value])



import pymysql
from notesecret import read_secret
from pymysql.cursors import DictCursor
from sqlalchemy import MetaData, Table, create_engine, delete, inspect, select
from sqlalchemy.dialects.mysql import insert

engine_dict = {}
meta_dict = {}


class create_conn(object):
    def __init__(self, database=None, *args, **kwargs):
        self.database = database
        self.instance = None

    def __enter__(self):
        self.instance = pymysql.connect(
            host=read_secret(cate1='notecoin', cate2='dataset', cate3='mysql', cate4='host'),
            user=read_secret(cate1='notecoin', cate2='dataset', cate3='mysql', cate4='user'),
            password=read_secret(cate1='notecoin', cate2='dataset', cate3='mysql', cate4='password'),
            database=self.database,
            charset="utf8mb4",
            cursorclass=DictCursor)
        self._handle = self.instance.cursor()
        return self._handle

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._handle.close()
        self.instance.close()


def create_database(db_name):
    with create_conn() as conn:
        conn.execute(f"CREATE DATABASE IF NOT EXISTS {db_name}")


def _get_uri(key=None):
    host = read_secret(cate1='notecoin', cate2='dataset', cate3='mysql', cate4='host')
    port = read_secret(cate1='notecoin', cate2='dataset', cate3='mysql', cate4='port') or '3306'
    user = read_secret(cate1='notecoin', cate2='dataset', cate3='mysql', cate4='user')
    password = read_secret(cate1='notecoin', cate2='dataset', cate3='mysql', cate4='password')
    db_name = 'notecoin' if key is None or len(key) == 0 else f'notecoin_{key.lower()}'
    create_database(db_name)
    return f"mysql+pymysql://{user}:{password}@{host}:{port}/{db_name}?charset=utf8"


def _init_engine_meta(key=None):
    engine = create_engine(_get_uri(key=key))
    engine_dict[key] = engine
    meta_dict[key] = MetaData(bind=engine)


def get_engine(key=None):
    if key not in engine_dict.keys():
        _init_engine_meta(key)
    return engine_dict[key]


def get_meta(key=None):
    if key not in meta_dict.keys():
        _init_engine_meta(key)
    return meta_dict[key]


class BaseTable:
    def __init__(self, table_name, db_suffix=None, *args, **kwargs):
        self.table_name = table_name
        self.table: Table = None
        self.engine = get_engine(db_suffix)
        self.meta = get_meta(db_suffix)

    def create(self):
        self.meta.create_all(self.engine)

    def select_all(self):
        return self.engine.execute(select([self.table]))

    def delete_all(self):
        return self.engine.execute(delete(self.table))

    def upsert(self, values):
        stmt = insert(self.table).values(values)
        primary_keys = [key.name for key in inspect(self.table).primary_key]
        update_dict = {c.name: c for c in stmt.inserted if
                       not c.primary_key and c.name not in ('gmt_create', 'gmt_modified')}
        stmt = stmt.on_duplicate_key_update(update_dict)
        self.engine.execute(stmt)
